from PIL import Image
import time
import math

"""
Génère le dégradé de couleurs.
Part de la couleur (r0, g0, b0)
puis passe par le blanc et va vers le noir.
seuil_fin permet de déterminer la proportion de couleurs
pour le dégradé blanc-noir.
"""
def faire_couleurs(nmax=255, seuil_fin=0.3):
    reste_fin = int(nmax*seuil_fin)
    r0, g0, b0 = (23, 50, 255)
    liste_couleurs = [(round(r0+(255-r0)*(n/(nmax-reste_fin))**(0.5)),
                       round(g0+(255-g0)*(n/(nmax-reste_fin))**(0.5)),
                       round(b0+(255-b0)*(n/(nmax-reste_fin))**(0.5))) for n in range(1, nmax+2-reste_fin)]
    r, g, b = liste_couleurs[-1]
    for i in range(1, reste_fin+1):
        coeff = (1-i/reste_fin)**1
        liste_couleurs.append((round(r*coeff), round(g*coeff), round(b*coeff)))
    return liste_couleurs

def visualiser_couleurs(liste_couleurs):
    l = len(liste_couleurs)
    h = 150
    nouvelle_image = Image.new("RGB", (l, h))
    for i in range(l):
        for j in range(h):
            nouvelle_image.putpixel((i, j), liste_couleurs[i])
    nouvelle_image.show()

def mandelbrot(nmax=255, centre = (-.5, 0), l=500, h=500, zoom=0, k=1, nom='mandelbrot.png', liste_couleurs=None):
    xc, yc = centre
    R = nmax**2
    l = l*k
    h = h*k
    echelle=(1/(0.5*l))*0.95**zoom
    xmin = xc - 0.5*l*echelle
    xmax = xc + 0.5*l*echelle
    ymin = yc - 0.5*h*echelle
    ymax = yc + 0.5*h*echelle
    dx = (xmax-xmin)/l
    dy = (ymax-ymin)/h
    nouvelle_image = Image.new("RGB", (l, h))
    #nouvelle_image = Image.new("HSV", (l, h))
    if liste_couleurs is None:
        liste_couleurs = faire_couleurs(255)#nmax=nmax)
        #visualiser_couleurs(liste_couleurs)
    #liste_couleurs = [(round(23+(255-22)*(n/nmax)**(0.33)), round(50+(255-50)*(n/nmax)**(0.1)), 255) for n in range(nmax+1)]
    n_min = nmax
    n_max = 0
    matrice = [[0 for _ in range(l)] for _ in range(h)]
    for i in range(l):
        #print(i)
        for j in range(h):
            x = xmin+i*dx
            y = ymax-j*dy
            c = complex(x, y)
            n = 0
            z = 0
            while n < nmax and abs(z) < 2:
            #while n < nmax and (z.imag**2 + z.real**2 < 4):
                z = z*z + c
                n += 1
            #if n < nmax:
            #    n = n + 1 - math.log(math.log2(abs(z)))
            matrice[i][j] = n
            if n < nmax:
                n_min = min(n_min, n)
                n_max = max(n_max, n)
    def calcule_indice_couleur(n):
        if n_min == n_max:
            return n
        else:
            return n%len(liste_couleurs)#round((n-n_min)/(n_max-n_min)*(len(liste_couleurs)-1))
    for i in range(h):
        for j in range(l):
            n = matrice[i][j]
            if n == nmax:
                nouvelle_image.putpixel((i, j), (0, 0, 0))
            else:
                #print(n,n_min,n_max,calcule_indice_couleur(n))
                nouvelle_image.putpixel((i, j), liste_couleurs[calcule_indice_couleur(n)])
    if k > 1:
        nouvelle_image = nouvelle_image.resize((l//k, h//k), Image.Resampling.LANCZOS)
    print((n_min, n_max), (xmin, xmax), (ymin, ymax))
    #nouvelle_image.save(nom)
    return nouvelle_image
    #nouvelle_image.show()

#visualiser_couleurs(faire_couleurs(nmax=1000))

"""
La partie animation est à revoir
pour que les vitesses de déplacement
soient moins importantes au début.
Et éventuellement pour permettre de placer plusieurs points de passages.
"""
def faire_animation(centre1, centre2, zoom1, zoom2, t, k, nmax, nom='mandelbrot.gif', liste_couleurs=None):
    x1, y1 = centre1
    x2, y2 = centre2
    liste_images = []
    #liste_couleurs = faire_couleurs(nmax=nmax)
    #visualiser_couleurs(liste_couleurs)
    for zoom in range(zoom1, zoom2+1):
        x = x2+(x1-x2)*0.9**zoom
        y = y2+(y1-y2)*0.9**zoom
        #zoom = zoom2+(zoom1-zoom2)*0.9**i
        debut = time.time()
        image = mandelbrot(nmax=nmax, centre=(x, y), l=t, h=t, zoom=zoom, k=k, liste_couleurs=liste_couleurs)
        print(zoom, time.time()-debut)
        liste_images.append(image)
    liste_images[0].save(nom,
               save_all=True, append_images=liste_images[1:], optimize=False, duration=60, loop=0)
    
nmax = 10000 # nombre d'itérations maximum avant de considérer que le point est dans l'ensemble.
# Plus nmax est grand, plus les frontières sont précises, mais plus c'est lent.
k = 1  # Calcule une image k fois plus grande pour lisser. Multiplie le temps de calcul par k^2.
t = 200 # taille en pixels. Sert pour la longueur et la largeur de l'image.

# Définition du centre et du niveau de zoom
# le niveau de zoom peut aller de -10 à environ 600
# Vous trouverez ci-dessous plusieurs valeurs "intéressantes"
centre, zoom = (-0.51, 0), 0  # image de base
#centre, zoom = (-1.764, 0), 78  # auto-similarité
#centre, zoom = (-1.78603, 0), 155  # autre auto-similarité
#centre, zoom = (-1.786343, 0), 230  # Il faut augmenter le nmax à 1000
#centre, zoom = (-1.7863453465, 0), 350  # Il faut augmenter le nmax à 1000
#centre, zoom = (-0.509999960095569,0.5150000450032913), 500  # Vous pouvez zoomer fort ici

animer = False # Pour décider si on fait une seule image ou une animation

debut = time.time()  # Pour mesurer le temps.
if animer:
    centre_init, zoom_init = (-.51, 0), -5  # le centre et le zoom de départ
    # On va zoomer jusqu'à arriver au centre final
    faire_animation(centre, centre, zoom_init, zoom, t, k, nmax, nom='mandelbrot.gif')
else:
    mandelbrot(nmax=nmax, centre=centre, l=t, h=t, zoom=zoom, k=k).show()
print(time.time()-debut)

"""
À la fin, les valeurs suivantes s'affichent :
* (n1, n2) : le nombre d'itération pous les points est compris entre n1 et n2.
             Si n1 est trop proche de n2, il faudrait peut-être augmenter nmax
* (x1, x2) : les abscisses sont comprises entre x1 et x2
* (y1, y2) : les ordonnées sont comprises entre y1 et y2
* temps    : le temps de calcul en secondes
"""