from PIL import Image, ImageDraw, ImageFont
import math
from initiation import *

TAILLE_FINALE = 800
TAILLE = TAILLE_FINALE * 4

NB_QUARTIERS = 80
ANGLE_QUARTIER = 360 / NB_QUARTIERS
COULEURS = {"0": (255, 255, 255), "1": (223, 106, 52)}
FOND = (100, 100, 100)

ALPHABET = " ABCDEFGHIJKLMNOPQRSTUVWXYZ',.?!"

GPS = "34 11 58 N 118 10 31 W"
GPS_LYCEE = "44 57 2 N 4 53 52 E"

# on ne met pas les 3 derniers 0
def codage(mot, taille=10):
    assert len(mot) < 8
    mot = mot.upper()
    code = ""
    for lettre in mot:
        if lettre in ALPHABET:
            nb = ALPHABET.index(lettre)
        else:
            nb = ALPHABET.index("?")
        binaire = conversion(nb)
        while len(binaire) < taille:
            binaire = "0" + binaire
        code = code + binaire
    return code

def decouper(texte, debut, longueur):
    resultat = ""
    for i in range(longueur):
        resultat = resultat + texte[debut+i]
    return resultat

def decodage(code):
    texte = ""
    position = 0
    while position < len(code):
        binaire = decouper(code, position, 10)
        position = position + 10
        nb = deconversion(binaire)
        if nb < len(ALPHABET):
            texte = texte + ALPHABET[nb]
        else:
            texte = texte + "?"
    return texte
        
def codage_gps(coord):
    liste = coord.split()
    assert len(liste)==8, "coordonnées GPS non valides"
    code = ""
    for i in range(len(liste)):
        if i%4 == 3:
            code = code + codage(liste[i])
        else:
            code = code + conversion(int(liste[i])).rjust(10, "0")
    return code

def decodage_gps(code):
    coord = ""
    for i in range(8):
        binaire = decouper(code, 10*i, 10)
        if i%4 == 3:
            valeur = decodage(binaire)
        else:
            valeur = str(deconversion(binaire))
        coord = coord + valeur
        if i < 7:
            coord = coord + " "
    return coord

def calculer_rayons(taille):
    r6 = taille//2
    r1 = r6/8
    r2 = 3*r1
    r3 = 5*r1
    r4 = 7*r1
    r5 = 7.5*r1
    return r1, r2, r3, r4, r5, r6

def polaire_vers_cartesien(r, angle, centre):
    x = centre[0] + r*math.cos(math.radians(angle))
    y = centre[1] - r*math.sin(math.radians(angle))
    return x, y

def dessiner_quartier(d, centre, r1, r2, angle, couleur, var1=0, var2=0, epaisseur=3):
    pt1 = polaire_vers_cartesien(r1+var1, angle, centre)
    pt2 = polaire_vers_cartesien(r2+var2, angle, centre)
    pt2b = polaire_vers_cartesien(r2, angle - 0.5*ANGLE_QUARTIER, centre)
    pt3 = polaire_vers_cartesien(r2-var2, angle - ANGLE_QUARTIER, centre)
    pt4 = polaire_vers_cartesien(r1-var1, angle - ANGLE_QUARTIER, centre)
    pt4b = polaire_vers_cartesien(r1, angle - 0.5*ANGLE_QUARTIER, centre)
    polygone = [pt1, pt2, pt2b, pt3, pt4, pt4b, pt1]
    d.polygon(polygone, fill=couleur, outline=None)
    d.line([pt1, pt2], width=epaisseur, fill=(0, 0, 0))
    d.line([pt3, pt4], width=epaisseur, fill=(0, 0, 0))
    if var1 == 0:
        d.arc([centre[0]-r1, centre[1]-r1, centre[0]+r1, centre[1]+r1], -angle, -angle+ANGLE_QUARTIER, fill=(0, 0, 0), width=epaisseur)
    else:
        d.line([pt1, pt4], width=epaisseur, fill=(0, 0, 0))
    if var2 == 0:
        d.arc([centre[0]-r2, centre[1]-r2, centre[0]+r2, centre[1]+r2], -angle, -angle+ANGLE_QUARTIER, fill=(0, 0, 0), width=epaisseur)
    else:
        d.line([pt2, pt3], width=epaisseur, fill=(0, 0, 0))

def dessiner_cercle(d, centre, r1, r2, code, var1=0, var2=0, quartier_debut=0):
    code = code + "000"
    for i in range(NB_QUARTIERS):
        if i < len(code):
            coul = COULEURS[code[i]]
        else:
            coul = COULEURS["1"]
        dessiner_quartier(d, centre, r1, r2, 90-(quartier_debut+i)*ANGLE_QUARTIER, coul, var1*((-1)**((quartier_debut+i)%2)), var2*((-1)**((quartier_debut+i)%2)))

def dessiner_parachute(texte, gps=GPS, quartier_debut=0, taille=TAILLE, taille_finale=TAILLE_FINALE):
    r1, r2, r3, r4, r5, r6 = calculer_rayons(taille)
    centre = (taille//2, taille//2)
    grande_image = Image.new("RGB", (taille, taille), FOND)
    d = ImageDraw.Draw(grande_image)
    texte = texte.upper()
    mots = texte.split()
    if len(mots) == 3 and len(mots[0]) < 8 and len(mots[1]) < 8 and len(mots[2]) < 8:
        code1 = codage(mots[0])
        code2 = codage(mots[1])
        code3 = codage(mots[2])
    else:
        assert len(texte)<22, "texte trop long"
        n = len(texte) // 3
        repartition = [n, n, len(texte)-2*n]
        if repartition[2] == 8:
            repartition[0] = repartition[0] + 1
            repartition[2] = repartition[2] - 1
        code1 = codage(decouper(texte, 0, repartition[0]))
        code2 = codage(decouper(texte, repartition[0], repartition[1]))
        code3 = codage(decouper(texte, repartition[0]+repartition[1], repartition[2]))
    debut = quartier_debut
    dessiner_cercle(d, centre, r1, r2, code1, var2=10, quartier_debut=debut)
    debut = debut + len(code1)
    dessiner_cercle(d, centre, r2, r3, code2, var1=10, var2=10, quartier_debut=debut)
    debut = debut + len(code2)
    dessiner_cercle(d, centre, r3, r4, code3, var1=10, quartier_debut=debut)
    debut = debut + len(code3)
    dessiner_cercle(d, centre, r5, r6, codage_gps(gps), quartier_debut=debut)
    image_finale = grande_image.resize((taille_finale, taille_finale))
    return image_finale

def sauver_image(image, nom="parachute.png"):
    image.save(nom)

def afficher_image(image):
    image.show()

def couleur_la_plus_proche(coul, couleurs):
    r, g, b = coul
    r0, g0, b0 = couleurs["0"]
    r1, g1, b1 = couleurs["1"]
    ecart0 = (r0-r)**2 + (g0-g)**2 + (b0-b)**2
    ecart1 = (r1-r)**2 + (g1-g)**2 + (b1-b)**2
    if ecart0 > ecart1:
        return "1"
    else:
        return "0"


def analyser_disque(image, r1, r2, centre, couleurs):
    r = (r1+r2) / 2
    vals = ""
    for i in range(NB_QUARTIERS):
        coul = image.getpixel(polaire_vers_cartesien(r, 90-(i+.5)*ANGLE_QUARTIER, centre))
        vals = vals + couleur_la_plus_proche(coul, couleurs)
    return vals

def trouver_debut(valeurs):
    i = 0
    n = len(valeurs)
    while i < n and valeurs[i] == "1":
        i = i + 1
    debut = i # au cas où le début est à la fin
    while i < n:
        compteur = 0
        while i < n and valeurs[i] == "1":
            compteur = compteur + 1
            i = i + 1
        if i < n and compteur >= 7:
            debut = i
        i = i + 1
    return debut

def extraire_code(valeurs, debut):
    code = ""
    i = (debut-10) % len(valeurs)
    while valeurs[i] == "1":
        i = (i-10) % len(valeurs)
    while i != debut:
        i = (i-1) % len(valeurs)
        code = valeurs[i] + code
    return code

def extraire_code_gps(valeurs, debut):
    code = ""
    for i in range(debut, len(valeurs)):
        code = code + valeurs[i]
    for i in range(0, debut):
        code = code + valeurs[i]
    return code
    
    
def decoder_parachute(nom="parachute.png", couleurs=COULEURS):
    image = Image.open(nom)
    taille = image.width
    centre = (taille//2, taille//2)
    r1, r2, r3, r4, r5, r6 = calculer_rayons(taille)
    valeurs1 = analyser_disque(image, r1, r2, centre, couleurs)
    debut = trouver_debut(valeurs1)
    code1 = extraire_code(valeurs1, debut)
    valeurs_gps = analyser_disque(image, r5, r6, centre, couleurs)
    code_gps = extraire_code_gps(valeurs_gps, debut)
    valeurs2 = analyser_disque(image, r2, r3, centre, couleurs)
    debut = (debut + len(code1)) % NB_QUARTIERS
    code2 = extraire_code(valeurs2, debut)
    valeurs3 = analyser_disque(image, r3, r4, centre, couleurs)
    debut = (debut + len(code2)) % NB_QUARTIERS
    code3 = extraire_code(valeurs3, debut)
    valeurs_gps = analyser_disque(image, r5, r6, centre, couleurs)
    debut = (debut + len(code3)) % NB_QUARTIERS
    code_gps = extraire_code_gps(valeurs_gps, debut)
    mot1 = decodage(code1)
    mot2 = decodage(code2)
    mot3 = decodage(code3)
    print("Si c'est 3 mots :")
    print(mot1 + " " + mot2 + " " + mot3)
    print("Sinon :")
    print(mot1 + mot2 + mot3)
    gps = decodage_gps(code_gps)
    print("Coordonnées GPS :")
    print(gps)